@@ -1,9 +1,8 @@
 use gdk;
 use glib::object::Cast;
-use glib::translate::ToGlib;
 use gtk::{
-    self, BoxExt, ContainerExt, GridExt, LabelExt, ProgressBarExt, ToggleButtonExt,
-    WidgetExt, GtkWindowExt,
+    self, AdjustmentExt, BoxExt, ContainerExt, GridExt, LabelExt, ProgressBarExt,
+    ScrolledWindowExt, ToggleButtonExt, WidgetExt, WidgetExtManual, GtkWindowExt,
 };
 use sysinfo::{self, ComponentExt, NetworkExt, ProcessorExt, SystemExt};
 
@@ -13,10 +12,17 @@ use std::rc::Rc;
 
 use graph::Graph;
 use notebook::NoteBook;
+use settings::Settings;
 use utils::{connect_graph, format_number, RotateVec};
 
-fn create_header(label_text: &str, parent_layout: &gtk::Box) -> gtk::CheckButton {
+fn create_header(
+    label_text: &str,
+    parent_layout: &gtk::Box,
+    display_graph: bool,
+) -> gtk::CheckButton {
     let check_box = gtk::CheckButton::new_with_label("Graph view");
+    check_box.set_active(display_graph);
+
     let label = gtk::Label::new(Some(label_text));
     let empty = gtk::Label::new(None);
     let grid = gtk::Grid::new();
@@ -68,11 +74,15 @@ pub struct DisplaySysInfo {
 }
 
 impl DisplaySysInfo {
-    pub fn new(sys: &Rc<RefCell<sysinfo::System>>, note: &mut NoteBook,
-               win: &gtk::ApplicationWindow) -> DisplaySysInfo {
+    pub fn new(
+        sys: &Rc<RefCell<sysinfo::System>>,
+        note: &mut NoteBook,
+        win: &gtk::ApplicationWindow,
+        settings: &Settings,
+    ) -> DisplaySysInfo {
         let vertical_layout = gtk::Box::new(gtk::Orientation::Vertical, 0);
         let mut procs = Vec::new();
-        let scroll = gtk::ScrolledWindow::new(None, None);
+        let scroll = gtk::ScrolledWindow::new(None::<&gtk::Adjustment>, None::<&gtk::Adjustment>);
         let mut components = vec!();
 
         // CPU
@@ -161,10 +171,10 @@ impl DisplaySysInfo {
 
         let non_graph_layout = gtk::Grid::new();
         non_graph_layout.set_column_homogeneous(true);
-        non_graph_layout.set_margin_right(5);
+        non_graph_layout.set_margin_end(5);
         let non_graph_layout2 = gtk::Grid::new();
         non_graph_layout2.set_column_homogeneous(true);
-        non_graph_layout2.set_margin_right(5);
+        non_graph_layout2.set_margin_start(5);
         let non_graph_layout3 = gtk::Box::new(gtk::Orientation::Vertical, 0);
         let non_graph_layout4 = gtk::Box::new(gtk::Orientation::Vertical, 0);
 
@@ -175,13 +185,13 @@ impl DisplaySysInfo {
             let p: &gtk::ProgressBar = &procs[0];
             let s = sys.borrow();
 
-            p.set_margin_right(5);
-            p.set_margin_left(5);
+            p.set_margin_end(5);
+            p.set_margin_start(5);
             p.set_show_text(true);
             let processor_list = s.get_processor_list();
             if !processor_list.is_empty() {
                 let pro = &processor_list[0];
-                p.set_text(format!("{:.2} %", pro.get_cpu_usage() * 100.).as_str());
+                p.set_text(Some(&format!("{:.2} %", pro.get_cpu_usage() * 100.)));
                 p.set_fraction(f64::from(pro.get_cpu_usage()));
             } else {
                 p.set_text(Some("0.0 %"));
@@ -194,14 +204,14 @@ impl DisplaySysInfo {
         //
         // PROCESS PART
         //
-        let check_box = create_header("Process usage", &vertical_layout);
+        let check_box = create_header("Process usage", &vertical_layout, settings.display_graph);
         for (i, pro) in sys.borrow().get_processor_list().iter().skip(1).enumerate() {
             let i = i + 1;
             procs.push(gtk::ProgressBar::new());
             let p: &gtk::ProgressBar = &procs[i];
-            let l = gtk::Label::new(format!("{}", i).as_str());
+            let l = gtk::Label::new(Some(&format!("{}", i)));
 
-            p.set_text(format!("{:.2} %", pro.get_cpu_usage() * 100.).as_str());
+            p.set_text(Some(&format!("{:.2} %", pro.get_cpu_usage() * 100.)));
             p.set_show_text(true);
             p.set_fraction(f64::from(pro.get_cpu_usage()));
             non_graph_layout.attach(&l, 0, i as i32 - 1, 1, 1);
@@ -216,7 +226,7 @@ impl DisplaySysInfo {
         //
         // MEMORY PART
         //
-        let check_box2 = create_header("Memory usage", &vertical_layout);
+        let check_box2 = create_header("Memory usage", &vertical_layout, settings.display_graph);
         let ram = create_progress_bar(&non_graph_layout2, 0, "RAM", "");
         let swap = create_progress_bar(&non_graph_layout2, 1, "Swap", "");
         vertical_layout.pack_start(&non_graph_layout2, false, false, 15);
@@ -232,13 +242,14 @@ impl DisplaySysInfo {
         // TEMPERATURES PART
         //
         if !sys.borrow().get_components_list().is_empty() {
-            check_box3 = Some(create_header("Components' temperature", &vertical_layout));
+            check_box3 = Some(create_header("Components' temperature", &vertical_layout,
+                                            settings.display_graph));
             for component in sys.borrow().get_components_list() {
                 let horizontal_layout = gtk::Box::new(gtk::Orientation::Horizontal, 10);
                 // TODO: add max and critical temperatures as well
-                let temp = gtk::Label::new(format!("{:.1} °C",
-                                                   component.get_temperature()).as_str());
-                horizontal_layout.pack_start(&gtk::Label::new(component.get_label()),
+                let temp = gtk::Label::new(Some(&format!("{:.1} °C",
+                                                        component.get_temperature())));
+                horizontal_layout.pack_start(&gtk::Label::new(Some(component.get_label())),
                                              true, false, 0);
                 horizontal_layout.pack_start(&temp, true, false, 0);
                 horizontal_layout.set_homogeneous(true);
@@ -257,20 +268,20 @@ impl DisplaySysInfo {
         //
         // NETWORK PART
         //
-        let check_box4 = create_header("Network usage", &vertical_layout);
+        let check_box4 = create_header("Network usage", &vertical_layout, settings.display_graph);
         // input data
-        let in_usage = gtk::Label::new(format_number(0).as_str());
+        let in_usage = gtk::Label::new(Some(&format_number(0)));
         let horizontal_layout = gtk::Box::new(gtk::Orientation::Horizontal, 10);
-        horizontal_layout.pack_start(&gtk::Label::new("Input data"), true, false, 0);
+        horizontal_layout.pack_start(&gtk::Label::new(Some("Input data")), true, false, 0);
         horizontal_layout.pack_start(&in_usage, true, false, 0);
         horizontal_layout.set_homogeneous(true);
         non_graph_layout4.add(&horizontal_layout);
         network_history.push(RotateVec::new(iter::repeat(0f64).take(61).collect()),
                              "Input data", None);
         // output data
-        let out_usage = gtk::Label::new(format_number(0).as_str());
+        let out_usage = gtk::Label::new(Some(&format_number(0)));
         let horizontal_layout = gtk::Box::new(gtk::Orientation::Horizontal, 10);
-        horizontal_layout.pack_start(&gtk::Label::new("Output data"), true, false, 0);
+        horizontal_layout.pack_start(&gtk::Label::new(Some("Output data")), true, false, 0);
         horizontal_layout.pack_start(&out_usage, true, false, 0);
         horizontal_layout.set_homogeneous(true);
         non_graph_layout4.add(&horizontal_layout);
@@ -296,14 +307,26 @@ impl DisplaySysInfo {
         scroll.add(&vertical_layout);
         note.create_tab("System usage", &scroll);
 
+        // It greatly improves the scrolling on the system information tab. No more clipping.
+        if let Some(adjustment) = scroll.get_vadjustment() {
+            adjustment.connect_value_changed(
+                clone!(cpu_usage_history, ram_usage_history, temperature_usage_history,
+                       network_history => move |_| {
+                cpu_usage_history.borrow().invalidate();
+                ram_usage_history.borrow().invalidate();
+                temperature_usage_history.borrow().invalidate();
+                network_history.borrow().invalidate();
+            }));
+        }
+
         let mut tmp = DisplaySysInfo {
             procs: Rc::new(RefCell::new(procs)),
             ram: ram.clone(),
             swap: swap.clone(),
             out_usage: out_usage.clone(),
             in_usage: in_usage.clone(),
-            vertical_layout: vertical_layout,
-            components: components,
+            vertical_layout,
+            components,
             cpu_usage_history: Rc::clone(&cpu_usage_history),
             ram_usage_history: Rc::clone(&ram_usage_history),
             ram_check_box: check_box.clone(),
@@ -313,9 +336,9 @@ impl DisplaySysInfo {
             network_history: Rc::clone(&network_history),
             network_check_box: check_box4.clone(),
         };
-        tmp.update_ram_display(&sys.borrow(), false);
+        tmp.update_system_info(&sys.borrow(), settings.display_fahrenheit);
 
-        win.add_events(gdk::EventType::Configure.to_glib() as i32);
+        win.add_events(gdk::EventMask::STRUCTURE_MASK);
         // TODO: ugly way to resize drawing area, I should find a better way
         win.connect_configure_event(move |w, _| {
             // To silence the annoying warning:
@@ -366,7 +389,7 @@ impl DisplaySysInfo {
         tmp
     }
 
-    pub fn update_ram_display(&mut self, sys: &sysinfo::System, display_fahrenheit: bool) {
+    pub fn update_system_info(&mut self, sys: &sysinfo::System, display_fahrenheit: bool) {
         let disp = |total, used| {
             if total < 100_000 {
                 format!("{} / {} kB", used, total)
@@ -382,7 +405,7 @@ impl DisplaySysInfo {
 
         let total_ram = sys.get_total_memory();
         let used = sys.get_used_memory();
-        self.ram.set_text(disp(total_ram, used).as_str());
+        self.ram.set_text(Some(&disp(total_ram, used)));
         if total_ram != 0 {
             self.ram.set_fraction(used as f64 / total_ram as f64);
         } else {
@@ -398,7 +421,7 @@ impl DisplaySysInfo {
 
         let total = ::std::cmp::max(sys.get_total_swap(), total_ram);
         let used = sys.get_used_swap();
-        self.swap.set_text(disp(sys.get_total_swap(), used).as_str());
+        self.swap.set_text(Some(&disp(sys.get_total_swap(), used)));
 
         let mut fraction = if total != 0 { used as f64 / total as f64 } else { 0f64 };
         if fraction.is_nan() {
@@ -413,6 +436,7 @@ impl DisplaySysInfo {
             }
         }
 
+        // temperature part
         let mut t = self.temperature_usage_history.borrow_mut();
         for (pos, (component, label)) in sys.get_components_list()
                                             .iter()
@@ -431,8 +455,9 @@ impl DisplaySysInfo {
                 label.set_text(&format!("{:.1} °C", component.get_temperature()));
             }
         }
+    }
 
-        // network part
+    pub fn update_network(&mut self, sys: &sysinfo::System) {
         let mut t = self.network_history.borrow_mut();
         self.in_usage.set_text(format_number(sys.get_network().get_income()).as_str());
         self.out_usage.set_text(format_number(sys.get_network().get_outcome()).as_str());
@@ -442,12 +467,12 @@ impl DisplaySysInfo {
         *t.data[1].get_mut(0).expect("cannot get data 1") = sys.get_network().get_outcome() as f64;
     }
 
-    pub fn update_process_display(&mut self, sys: &sysinfo::System) {
+    pub fn update_system_info_display(&mut self, sys: &sysinfo::System) {
         let v = &*self.procs.borrow_mut();
         let h = &mut *self.cpu_usage_history.borrow_mut();
 
         for (i, pro) in sys.get_processor_list().iter().enumerate() {
-            v[i].set_text(format!("{:.1} %", pro.get_cpu_usage() * 100.).as_str());
+            v[i].set_text(Some(&format!("{:.1} %", pro.get_cpu_usage() * 100.)));
             v[i].set_show_text(true);
             v[i].set_fraction(f64::from(pro.get_cpu_usage()));
             if i > 0 {
